package commands

import (
	"fmt"
	"encoding/json"
	"os"
	"strings"
	"path/filepath"
	"datasmith/utils"
	"gopkg.in/yaml.v2"
	"regexp"
	"strconv"
	"time"
	"bytes"
	"bufio"
)

type Field struct {
	Name         string      `json:"name"`
	Type         string      `json:"type"`
	PrimaryKey   bool        `json:"primary_key,omitempty"`
	ForeignKey   string      `json:"foreign_key,omitempty"`
	Unique       bool        `json:"unique,omitempty"`
	NotNull      bool        `json:"not_null,omitempty"`
	AutoIncrement bool       `json:"auto_increment,omitempty"`
	DefaultValue interface{} `json:"default_value,omitempty"`
}

type TableModel struct {
	Fields []Field `json:"fields"`
}

type DatasmithConfig struct {
	Name string 									`yaml:"name"`
	Version string 								`yaml:"version"`
	CreatedAt string 						`yaml:"created_at"`
	DbType string              		`yaml:"database_type"`
	Tables map[string]TableModel 	`yaml:"tables"`
}



func Add(tableName string, model string) {
	utils.PromptIfEmpty(&tableName, "Enter the name of the table: ")

	projectDir := "." // Assuming current directory is the project directory
	config, err := getDbTypeFromConfig(projectDir)
	if err != nil {
		fmt.Printf("Error loading config: %v\n", err)
		return
	}

	if _, exists := config.Tables[tableName]; exists {
		fmt.Printf("Table '%s' already exists in the project\n", tableName)
		return
	}

	var tableModel TableModel
	if model != "" {
		if err := json.Unmarshal([]byte(model), &tableModel); err != nil {
			fmt.Printf("Error parsing JSON model: %v\n", err)
			return
		}
		if err := validateModel(tableModel); err != nil {
			fmt.Printf("Invalid model: %v\n", err)
			return
		}
	} else {
		tableModel = promptForTableModel()
	}

	slug := utils.Slugify(tableName)

	// Update datasmith.yaml config and save the new table definition
	// Initialize Tables map if it's nil
	if config.Tables == nil {
		config.Tables = make(map[string]TableModel)
	}
	config.Tables[slug] = tableModel

	// Bump the version
	newVersion, err := bumpVersion(config.Version)
	if err != nil {
		fmt.Printf("Error bumping version: %v\n", err)
		return
	} else {
		fmt.Printf("Bumped version to %s\n", newVersion)
	}
	config.Version = newVersion

	if err := saveConfig(projectDir, config); err != nil {
		fmt.Printf("Error saving config: %v\n", err)
	}

	// Create the table file under sql directory
	createTableFile(projectDir, tableName, config.DbType, tableModel)

	// Append import command to import-sql.sh
	if err := appendToImportScript(projectDir, tableName, config.DbType); err != nil {
		fmt.Printf("Error appending to import-sql.sh: %v\n", err)
	} else {
		fmt.Printf("Appended import command for table '%s' to import-sql.sh\n", tableName)
	}

	// Append table description to database.dbml
	if err := appendToDBMLFile(projectDir, tableName, tableModel); err != nil {
		fmt.Printf("Error appending to database.dbml: %v\n", err)
	} else {
		fmt.Printf("Appended table '%s' to database.dbml\n", tableName)
	}

	// Update CHANGELOG
	if err := updateChangelog(projectDir, newVersion, tableName); err != nil {
		fmt.Printf("Error updating CHANGELOG: %v\n", err)
	} else {
		fmt.Printf("Updated CHANGELOG with version '%s'\n", newVersion)
	}

	// Update README.md with the new table information
	if err := updateReadme(projectDir, tableName, tableModel); err != nil {
		fmt.Printf("Error updating README.md: %v\n", err)
	} else {
		fmt.Printf("Updated README.md with table '%s'\n", tableName)
	}
	fmt.Print("\n")
	fmt.Print("------------")
	fmt.Print("\n")
	fmt.Printf("Added new table '%s' to the project\n", slug)
}

func getDbTypeFromConfig(projectDir string) (DatasmithConfig, error) {
	configFilePath := filepath.Join(projectDir, "datasmith.yaml")
	file, err := os.Open(configFilePath)
	if err != nil {
		return DatasmithConfig{}, fmt.Errorf("error opening config file: %v", err)
	}
	defer file.Close()

	var config DatasmithConfig
	decoder := yaml.NewDecoder(file)
	if err := decoder.Decode(&config); err != nil {
		return DatasmithConfig{}, fmt.Errorf("error decoding config file: %v", err)
	}

	// Initialize Tables map if it's nil
	if config.Tables == nil {
		config.Tables = make(map[string]TableModel)
	}

	fmt.Printf("Loaded config for %s:%s\n", config.Name, config.Version)

	return config, nil
}

func saveConfig(projectDir string, config DatasmithConfig) error {
	configFilePath := filepath.Join(projectDir, "datasmith.yaml")
	file, err := os.Create(configFilePath)
	if err != nil {
		return fmt.Errorf("error creating config file: %v", err)
	}
	defer file.Close()

	encoder := yaml.NewEncoder(file)
	if err := encoder.Encode(&config); err != nil {
		return fmt.Errorf("error encoding config file: %v", err)
	}

	return nil
}

func createTableFile(projectDir, tableName, dbType string, tableModel TableModel) {
	slug := utils.Slugify(tableName)
	sqlFilePath := filepath.Join(projectDir, "sql", fmt.Sprintf("%s.sql", slug))

	file, err := os.Create(sqlFilePath)
	if err != nil {
		fmt.Printf("Error creating SQL file: %v\n", err)
		return
	}
	defer file.Close()

	if err := generateSQL(file, tableName, dbType, tableModel); err != nil {
		fmt.Printf("Error generating SQL: %v\n", err)
		return
	}

	fmt.Printf("Created file: %s\n", sqlFilePath)
}


func generateSQL(file *os.File, tableName, dbType string, tableModel TableModel) error {
	switch dbType {
	case "mysql":
		return generateMySQLSQL(file, tableName, tableModel)
	case "postgres":
		return generatePostgreSQLSQL(file, tableName, tableModel)
	default:
		return fmt.Errorf("unsupported database type: %s", dbType)
	}
}

func generateMySQLSQL(file *os.File, tableName string, tableModel TableModel) error {
	var sql strings.Builder

	sql.WriteString(fmt.Sprintf("CREATE TABLE %s (\n", tableName))

	for i, field := range tableModel.Fields {
		sql.WriteString(fmt.Sprintf("  %s %s", field.Name, field.Type))

		if field.PrimaryKey {
			sql.WriteString(" PRIMARY KEY")
		}
		if field.Unique {
			sql.WriteString(" UNIQUE")
		}
		if field.NotNull {
			sql.WriteString(" NOT NULL")
		}
		if field.AutoIncrement {
			sql.WriteString(" AUTO_INCREMENT")
		}
		if field.DefaultValue != nil {
			sql.WriteString(fmt.Sprintf(" DEFAULT '%v'", field.DefaultValue))
		}
		if field.ForeignKey != "" {
			sql.WriteString(fmt.Sprintf(",\n  FOREIGN KEY (%s) REFERENCES %s", field.Name, field.ForeignKey))
		}

		if i < len(tableModel.Fields)-1 {
			sql.WriteString(",\n")
		}
	}

	sql.WriteString("\n);\n")

	_, err := file.WriteString(sql.String())
	return err
}

func generatePostgreSQLSQL(file *os.File, tableName string, tableModel TableModel) error {
	var sql strings.Builder

	sql.WriteString(fmt.Sprintf("CREATE TABLE %s (\n", tableName))

	for i, field := range tableModel.Fields {
		sql.WriteString(fmt.Sprintf("  %s %s", field.Name, field.Type))

		if field.PrimaryKey {
			sql.WriteString(" PRIMARY KEY")
		}
		if field.Unique {
			sql.WriteString(" UNIQUE")
		}
		if field.NotNull {
			sql.WriteString(" NOT NULL")
		}
		if field.AutoIncrement {
			sql.WriteString(" SERIAL")
		}
		if field.DefaultValue != nil {
			sql.WriteString(fmt.Sprintf(" DEFAULT '%v'", field.DefaultValue))
		}
		if field.ForeignKey != "" {
			sql.WriteString(fmt.Sprintf(",\n  FOREIGN KEY (%s) REFERENCES %s", field.Name, field.ForeignKey))
		}

		if i < len(tableModel.Fields)-1 {
			sql.WriteString(",\n")
		}
	}

	sql.WriteString("\n);\n")

	_, err := file.WriteString(sql.String())
	return err
}

func updateReadme(projectDir, tableName string, tableModel TableModel) error {
	readmeFilePath := filepath.Join(projectDir, "README.md")

	// Read the existing README.md content
	file, err := os.Open(readmeFilePath)
	if err != nil {
		return fmt.Errorf("error reading README.md file: %v", err)
	}
	defer file.Close()

	var buffer bytes.Buffer
	scanner := bufio.NewScanner(file)
	inTablesSection := false
	inMermaidSection := false
	inDbmlSection := false
	var tablesSection []string
	var mermaidSection []string
	var dbmlSection []string

	for scanner.Scan() {
		line := scanner.Text()
		trimmedLine := strings.TrimSpace(line)

		if strings.HasPrefix(trimmedLine, "```mermaid") {
			inMermaidSection = true
			buffer.WriteString(line + "\n")
			continue
		}

		if strings.HasPrefix(trimmedLine, "```dbml") {
			inDbmlSection = true
			buffer.WriteString(line + "\n")
			continue
		}

		if inMermaidSection && strings.HasPrefix(trimmedLine, "```") {
			inMermaidSection = false
			mermaidSection = append(mermaidSection, generateMermaidDiagram(tableName, tableModel))
			buffer.WriteString(strings.Join(mermaidSection, "\n") + "\n")
			buffer.WriteString(line + "\n")
			continue
		}

		if inDbmlSection && strings.HasPrefix(trimmedLine, "```") {
			inDbmlSection = false
			dbmlSection = append(dbmlSection, generateDBML(tableName, tableModel))
			buffer.WriteString(strings.Join(dbmlSection, "\n") + "\n")
			buffer.WriteString(line + "\n")
			continue
		}

		if strings.Contains(trimmedLine, "NO TABLES GIVEN") {
			inTablesSection = true
			buffer.WriteString(strings.Replace(line, "NO TABLES GIVEN", "Tables:\n", 1))
			continue
		}

		if inTablesSection && strings.HasPrefix(trimmedLine, "```") {
			inTablesSection = false
			buffer.WriteString(strings.Join(tablesSection, "\n") + "\n")
			buffer.WriteString(line + "\n")
			continue
		}

		if inTablesSection {
			tablesSection = append(tablesSection, "- "+tableName)
		}

		buffer.WriteString(line + "\n")
	}

	// Append table if it was not in any sections
	if inTablesSection {
		buffer.WriteString(strings.Join(tablesSection, "\n") + "\n")
	}
	if inMermaidSection {
		mermaidSection = append(mermaidSection, generateMermaidDiagram(tableName, tableModel))
		buffer.WriteString(strings.Join(mermaidSection, "\n") + "\n")
	}
	if inDbmlSection {
		dbmlSection = append(dbmlSection, generateDBML(tableName, tableModel))
		buffer.WriteString(strings.Join(dbmlSection, "\n") + "\n")
	}

	if err := scanner.Err(); err != nil {
		return fmt.Errorf("error scanning README.md file: %v", err)
	}

	// Write the updated content back to the README.md file
	if err := os.WriteFile(readmeFilePath, buffer.Bytes(), 0644); err != nil {
		return fmt.Errorf("error writing updated README.md file: %v", err)
	}

	return nil
}

func generateMermaidDiagram(tableName string, tableModel TableModel) string {
	var mermaid strings.Builder

	mermaid.WriteString(fmt.Sprintf("    %s {\n", tableName))

	for _, field := range tableModel.Fields {
		mermaid.WriteString(fmt.Sprintf("        %s %s\n", field.Name, field.Type))
	}

	mermaid.WriteString("    }\n")

	return mermaid.String()
}
func generateDBML(tableName string, tableModel TableModel) string {
	var dbml strings.Builder

	dbml.WriteString(fmt.Sprintf("Table %s {\n", tableName))

	for _, field := range tableModel.Fields {
		dbml.WriteString(fmt.Sprintf("  %s %s", field.Name, field.Type))

		if field.PrimaryKey {
			dbml.WriteString(" [pk]")
		}
		if field.Unique {
			dbml.WriteString(" [unique]")
		}
		if field.NotNull {
			dbml.WriteString(" [not null]")
		}
		if field.AutoIncrement {
			dbml.WriteString(" [autoincrement]")
		}
		if field.DefaultValue != nil {
			dbml.WriteString(fmt.Sprintf(" [default: '%v']", field.DefaultValue))
		}

		dbml.WriteString("\n")
	}

	dbml.WriteString("}\n")

	return dbml.String()
}

func appendToDBMLFile(projectDir, tableName string, tableModel TableModel) error {
	dbmlFilePath := filepath.Join(projectDir, "database.dbml")
	file, err := os.OpenFile(dbmlFilePath, os.O_APPEND|os.O_WRONLY, os.ModeAppend)
	if err != nil {
		return fmt.Errorf("error opening database.dbml file: %v", err)
	}
	defer file.Close()

	dbmlContent := generateDBML(tableName, tableModel)
	if _, err := file.WriteString(dbmlContent); err != nil {
		return fmt.Errorf("error writing to database.dbml file: %v", err)
	}

	return nil
}

func updateChangelog(projectDir, newVersion, tableName string) error {
	changelogFilePath := filepath.Join(projectDir, "CHANGELOG.md")
	
	// Read the existing CHANGELOG content
	file, err := os.ReadFile(changelogFilePath)
	if err != nil {
		return fmt.Errorf("error reading CHANGELOG.md file: %v", err)
	}

	content := string(file)

	// Create the new changelog entry
	newEntry := fmt.Sprintf("## [%s] - %s\n\n### Added\n\n- Added table `%s`\n\n", newVersion, time.Now().Format("2006-01-02"), tableName)

	// Insert the new entry after the "## [Unreleased]" line
	insertPos := strings.Index(content, "## [Unreleased]")
	if insertPos == -1 {
		// If "## [Unreleased]" not found, prepend the new entry
		content = newEntry + content
	} else {
		insertPos += len("## [Unreleased]\n\n")
		content = content[:insertPos] + newEntry + content[insertPos:]
	}

	// Write the updated content back to the CHANGELOG file
	if err := os.WriteFile(changelogFilePath, []byte(content), 0644); err != nil {
		return fmt.Errorf("error writing to CHANGELOG.md file: %v", err)
	}

	return nil
}

func appendToImportScript(projectDir, tableName, dbType string) error {
	scriptFilePath := filepath.Join(projectDir, "import-sql.sh")
	tableSQLFile := fmt.Sprintf("/tmp/%s.sql", utils.Slugify(tableName))

	var importCommand string
	switch dbType {
	case "mysql":
		importCommand = "\n" + fmt.Sprintf(`echo "Importing table %s"`, tableName) + "\n" +
			fmt.Sprintf(`mariadb -u root -p"$MARIADB_ROOT_PASSWORD" "$DB_DATABASE" < %s`, tableSQLFile) + "\n"
	case "postgres":
		importCommand = "\n" + fmt.Sprintf(`echo "Importing table %s"`, tableName) + "\n" +
			fmt.Sprintf(`psql -U "$DB_USER" -d "$DB_DATABASE" -f %s`, tableSQLFile) + "\n"
	default:
		return fmt.Errorf("unsupported database type: %s", dbType)
	}

	file, err := os.OpenFile(scriptFilePath, os.O_APPEND|os.O_WRONLY, os.ModeAppend)
	if err != nil {
		return fmt.Errorf("error opening import-sql.sh file: %v", err)
	}
	defer file.Close()

	if _, err := file.WriteString(importCommand); err != nil {
		return fmt.Errorf("error writing to import-sql.sh file: %v", err)
	}

	return nil
}

func validateModel(model TableModel) error {
	fieldNames := make(map[string]bool)

	for _, field := range model.Fields {
		// Check if field name is empty
		if field.Name == "" {
			return fmt.Errorf("field name cannot be empty")
		}

		// Check for duplicate field names
		if _, exists := fieldNames[field.Name]; exists {
			return fmt.Errorf("duplicate field name: %s", field.Name)
		}
		fieldNames[field.Name] = true

		// Check if field type is valid
		if !isValidType(field.Type) {
			return fmt.Errorf("invalid field type: %s", field.Type)
		}

		// Check if primary key is properly set
		if field.PrimaryKey {
			if field.ForeignKey != "" {
				return fmt.Errorf("field %s cannot be both a primary key and a foreign key", field.Name)
			}
			if field.AutoIncrement && field.Type != "INT" {
				return fmt.Errorf("auto increment can only be used with INT type for field %s", field.Name)
			}
		}

		// Check for incompatible settings
		if field.AutoIncrement && !field.PrimaryKey {
			return fmt.Errorf("auto increment can only be used with a primary key for field %s", field.Name)
		}

		// Add any additional validation rules as needed
	}

	return nil
}

func isValidType(fieldType string) bool {
	for _, t := range FieldTypes {
		if t == fieldType {
			return true
		}
	}
	return false
}

func promptForTableModel() TableModel {
	var fields []Field

	for {
		var field Field
		utils.PromptIfEmpty(&field.Name, "Enter field name: ")
		field.Type = promptForFieldType()

		field.PrimaryKey = utils.PromptForBool("Is this a primary key? (y/n): ") 
		if !field.PrimaryKey {
			utils.PromptIfEmpty(&field.ForeignKey, "Enter foreign key (leave empty if not applicable): ")
		}
		field.Unique = utils.PromptForBool("Is this field unique? (y/n): ") 
		field.NotNull = utils.PromptForBool("Is this field not null? (y/n): ") 
		field.AutoIncrement = utils.PromptForBool("Is this field auto-increment? (y/n): ")
		field.DefaultValue = utils.PromptForString("Enter default value (leave empty if not applicable): ")

		fields = append(fields, field)

		if !utils.PromptForBool("Do you want to add another field? (y/n): ") {
			break
		}
	}

	return TableModel{Fields: fields}
}

func promptForFieldType() string {
	options := make([]utils.MenuOption, len(FieldTypes))
	for i, t := range FieldTypes {
		options[i] = utils.MenuOption{Display: t, Value: t}
	}

	selected, err := utils.SelectMenu(options, "Select field type:")
	if err != nil {
		fmt.Printf("Error selecting field type: %v\n", err)
		return ""
	}

	return selected
}

// Function to bump the version by a minor version
func bumpVersion(version string) (string, error) {
	re := regexp.MustCompile(`^(\d+)\.(\d+)\.(\d+)$`)
	matches := re.FindStringSubmatch(version)
	if matches == nil {
		return "", fmt.Errorf("invalid version format: %s", version)
	}

	major, _ := strconv.Atoi(matches[1])
	minor, _ := strconv.Atoi(matches[2])
	patch, _ := strconv.Atoi(matches[3])

	minor++
	newVersion := fmt.Sprintf("%d.%d.%d", major, minor, patch)
	return newVersion, nil
}

// Help returns the help information for the add command
func AddHelp() string {
	return "add <tablename> [--model <json_model>]: Add a table to the project."
}